@@ -32,9 +32,9 @@ def main():
     
     #pixel density of image to plot
     ratio = 0.8
-    hpix = 600.       #number of horizontal pixels
-    vpix = ratio*hpix #number of vertical pixels
-    img_res = (int(vpix),int(hpix))
+    hpix = 600.       #number of horizontal pixels  E-W
+    vpix = ratio*hpix #number of vertical pixels    S-N
+    img_res = (int(hpix),int(vpix))
     
     ##define Albers projection and extend of map
     #Obtained through trial and error for good fit of the mdel grid being plotted
